/**********************************************************************

Automate.h
COPYRIGHT (c) 2016 Dominique Bultez
Automate pour un Va et Vient en DCC

---------------
Les variables globales sont en MAJUSCULE et précédées de la lettre "g"
---------------
variable ETAT (gETAT) :

  INIT = 0 non initialisé (la machine ne doit pas être posée sur les zones de gare aux extrémités pour permettre l'état EXPLORE
      découvrir l'adresse DCC : gADDRESS_DCC
  EXPLORE = 1 faire rouler en marche avant (vitesse 30 ou potar), enregistrer la zone de depart et armer timeout
      detecter le sens de marche par détection de la gare d'arrivée
      si gare 2 marche_avant = Gare1->Gare2, sinon marche_avant = Gare2->Gare1
      arrêt immédiat de la machine + arret sur timeout si gare non trouvée
  ARRET = 2 initialiser le trajet (gare1->2 ou gare2->1 selon le point de départ et sens marche arrière/avant)
      démarrer une tempo d'arrêt en gare puis attente fin tempo d'attente en gare
  ACCELERE = 3 acceleration en marche arrière/avant jusqu'à vitesse de consigne
      algo : incrément linéaire avec periodicité constante
      periodicité et incrément déterminés pour atteindre la vitesse palier avant la fin des zones A ou C
  PALIER = 4 vitesse palier = consigne (potar ou encodeur) jusqu'à détecteur de zone C ou A avant gare 
      armement d'une tempo de mesure du temps de traversée de chaque zone centrale B
      mesure de vitesse réelle a la sortie de zone B et correction du cran de vitesse pour atteindre consigne
  RALENTI = 5 deceleration jusqu'à vitesse min ou détecteur gare2 ou gare1 + arret sur timeout
      algo : deceleration lineaire avec decrément variable et calcul de correction du décrément
      - Si vitesse min atteinte avant gare : mesure du temps jusqu'à gare et correction de la décélération (diminuer décrément)
      - Si gare atteinte avant vitesse min : relevé du cran à l'entrée en gare et correction de la décélération (augmenter décrément)
  ARRET = 2 initialiser le sens du trajet et démarrer une tempo d'arrêt en gare
  
  Puis ACCELERE, PALIER, RALENTI, ARRET, ...
----------------
Detection de zones
  ZGARE1=0
  ZONEA=1
  ZONEB=2
  ZONEC=3
  ZGARE2=5
  A chaque disparition de détection de zone (et en sortie de INIT) on arme une tempo de perte de train éventuel.
  
byte occupation = 0x000xxxxx
Position du bit = n° de zone
Valeur du bit = 1 occupé, O libre
----------------

**********************************************************************/

/////////////////////////////////////////////////////////////////////////////////////

//#ifndef Automate_h
//#define Automate_h
//#include "Arduino.h"

enum {              // nom des états
  INIT=0,
  EXPLORE=1,
  ARRET=2,
  ACCELERE=3,
  PALIER=4,
  RALENTI=5
};

int gETAT = 0;      // etat de l'automate

enum {
  ZGARE1=0,
  ZONEA=1,
  ZONEB=2,
  ZONEC=3,
  ZGARE2=4
};

const char * nomZone[ZGARE2+1]  = {     // noms zones sur LCD
  "Gar1 ",
  "ZonA ",
  "ZonB ",
  "ZonC ",
  "Gar2 "
};



#define  ZONE_SAMPLE_TIME     100       // ms
#define  DEBOUNCE_SAMPLE_TIME 100       // ms
#define  AUTOMATE_TIMEOUT     120000    // 2 minutes
#define  NULCURRENT_TIME      10000     // 10 secondes
#define  DISTANCE_MODULE      120       // cm
#define  VITESSE_MIN          10        //cran

#define  BOK                A1    // bouton OK
#define  STARTSTOP          A2    //inter Marche / Arret
#define  LUMIERE            A3    //inter FEUX
#define  PINGARE1           5     //capteur input
#define  PINZONEA           6     //capteur input
#define  PINZONEB           7     //capteur input
#define  PINZONEC           8     //capteur input
#define  PINGARE2           9     //capteur input
#define  LEDROUGE           11    //led rouge output
#define  ENC1               2     //encoder input
#define  ENC2               4     //encoder input

#define  ARRETENGARE        100   // x0,1s soit 10 secondes
#define  ACCELERATION       5     // incrément de cran de vitesse
#define  DECELERATION       5     // décrement de cran de vitesse

struct ConfigEEprom {
  unsigned int vers;              // version logiciel
  unsigned long dist;             // = DISTANCE_MODULE; en cm
  int arret;                      // = ARRETENGARE;     en s
  int adresse;                    // = 3; ADRESSE par DEFAUT
  int vmin;                       // = vitesse minimum
};



//////////////////////////// AUTOMATE ///////////////////////////////
struct Automate{
  int auto_etat;                  // INIT, EXPLORE, ARRET, ACCELERE, PALIER, RALENTI
  unsigned long arretengare;      // decompteur temps d'arret en gare en ms
  unsigned long timeout, timeseconde; // timecheck
  unsigned long timeLR;           // tempo clignotement Led Rouge
  bool auto_seconde;              // tempo seconde écoulée
  bool auto_ERR;                  // erreur automate
  bool LRclignote;                // true = Led Rouge clignotante
  // fonctions
  Automate(int);
  void runauto();
};

Automate::Automate(int ae) {
  auto_etat = INIT;
  timeseconde=millis();
  timeLR=millis();
  arretengare = ARRETENGARE;     // tempo d'arret en gare configurable
  auto_ERR = false;
  LRclignote = false;
}

void Automate::runauto() {
  if (millis()-timeLR > 500) {  // clignotement LED ROUGE
    if (LRclignote) {  // clignotement LED ROUGE
      digitalWrite(LEDROUGE, !digitalRead(LEDROUGE));
      timeLR = millis();
    }
  } 
  if (millis()-timeseconde > 1000) { // base de temps seconde
    auto_seconde = true;
  }
  if (millis()-timeout > AUTOMATE_TIMEOUT) { // time-out
    auto_ERR = true;
  }
}


//////////////////////////// TRAIN ///////////////////////////////

struct Train{
  int adresse_dcc;            // pour la commande DCC
  bool sens1vers2;            // true si gare 1-> gare 2
  bool sens;                  // pour la commande DCC
  bool lumiere;               // pour la commande DCC
  bool troplent;              // true si Vmin atteinte pendant RALENTI
  int zone;                   // la zone ou il est
  int cran_vitesse;           // cran DCC en cours
  int cran_consigne;          // cran DCC à la consigne
  int consigne;               // en km/H au potar et LCD
  int vitesseVraie;           // en km/h
  int acceleration;           // en crans
  int deceleration1;          // en crans vers gare 1
  int deceleration2;          // en crans vers gare 2
  Train(int);
  void roule();               // fonction pour avancer/reculer
  void stoppe();              // fonction pour s'arreter
  void feux(bool);            // commande des lumieres
  void eteint();
  void allume();
};

Train::Train(int a) {
  adresse_dcc = a;
  sens = true;
  cran_vitesse = 30;
  lumiere = false;
  acceleration = ACCELERATION;
  deceleration1 = DECELERATION;
  deceleration2 = DECELERATION;
  cran_consigne = 60;
}

//////////////////////////// ZONES ///////////////////////////////

struct ZoneMonitor {
  unsigned long zoneTime, zoneTimeA, zoneTimeAG1, zoneTimeCG2, zoneTimeC;
  int pin1, pinA, pinB, pinC, pin2;
  byte occupation;
  ZoneMonitor(int, int, int, int, int);
  void Init();
  bool checkZoneTime();
  byte checkZone();
};

ZoneMonitor::ZoneMonitor(int pin1, int pina, int pinb, int pinc,int pin2){
    this->pin1=pin1;
    this->pinA=pina;
    this->pinB=pinb;
    this->pinC=pinc;
    this->pin2=pin2;
    occupation = 0;
} // ZoneMonitor::ZoneMonitor

void ZoneMonitor::Init() {
    pinMode(pin1,INPUT_PULLUP);
    pinMode(pinA,INPUT_PULLUP);
    pinMode(pinB,INPUT_PULLUP);
    pinMode(pinC,INPUT_PULLUP);
    pinMode(pin2,INPUT_PULLUP);
    bitWrite(occupation, ZGARE1, !digitalRead(pin1)); // LOW = occupé
    bitWrite(occupation, ZONEA, !digitalRead(pinA)); // LOW = occupé
    bitWrite(occupation, ZONEB, !digitalRead(pinB)); // LOW = occupé
    bitWrite(occupation, ZONEC, !digitalRead(pinC)); // LOW = occupé
    bitWrite(occupation, ZGARE2, !digitalRead(pin2)); // LOW = occupé    
    zoneTime = millis();  
}

bool ZoneMonitor::checkZoneTime() {
  if (millis()-zoneTime < ZONE_SAMPLE_TIME)    // test préiodique
    return(false);
  zoneTime = millis();   
  return(true);  
} // ZoneMonitor::checkTime

byte ZoneMonitor::checkZone() {      // détection de changement d'état
  bitWrite(occupation, ZGARE1, !digitalRead(pin1)); // LOW = occupé -> un bit 1
  bitWrite(occupation, ZONEA, !digitalRead(pinA));
  bitWrite(occupation, ZONEB, !digitalRead(pinB));
  bitWrite(occupation, ZONEC, !digitalRead(pinC));
  bitWrite(occupation, ZGARE2, !digitalRead(pin2));
  return(occupation);
}


//-------------------------------------------------------------------


