/////////////////////////////////////////////////////////////////////////////////////////
// FONCTION DE CONFIGURATION APPELÉE UNIQUEMENT AU DÉMARRAGE EN LAISSANT LE BOUTON APPUYÉ
/////////////////////////////////////////////////////////////////////////////////////////

void afficheLCD(int val) {
    lcd.setCursor(8,1);
    if (val < 100) lcd.print(" ");
    if (val < 10) lcd.print(" ");
    lcd.print(val);
}

int configure(int val) {
  afficheLCD(val);
  oldPosition = myEnc.read()/4;

  do {
  long newPosition = myEnc.read()/4;
    if (newPosition != oldPosition) {  
      if ( newPosition > oldPosition) val++;  else  val--;
      oldPosition = newPosition; 
      afficheLCD(val);
      delay(100);
    }
  } while (digitalRead(BOK) == true);    // attente appui sur bouton
  
  do {delay(100);} while (digitalRead(BOK) == false); // antirebond
  // attente relachement bouton
  while (digitalRead(BOK) == false) {delay(100);}
  return(val);
}

//////// Configuration de la distance de la zone de mesure de vitesse, de la durée d'arret en gare 
//////// et de l'adresse DCC par défaut en cas de non reconaissance de l'adresse

void configuration() {
  int valeur;
  
  EEPROM.get(0, eeConfig);              // lecture record eeConfig en EEPROM
  #ifdef DEBUG
  Serial.println(eeConfig.vers);
  Serial.println(eeConfig.dist);
  Serial.println(eeConfig.arret);
  Serial.println(eeConfig.adresse);
  Serial.println(eeConfig.vmin);
  #endif
  if (eeConfig.vers == 0xFFFF) {        // si l'EEPROM est vierge, on écrit les valeurs par défaut
    eeConfig.dist = DISTANCE_MODULE;
    eeConfig.arret = ARRETENGARE;
    eeConfig.adresse = 3;
    eeConfig.vers = EEPVERSION;
    eeConfig.vmin = VITESSE_MIN;
    EEPROM.put(0, eeConfig);
  }
  if (digitalRead(BOK) == false) {      // mode configuration en appuyant sur BOK au démarrage
    lcd.clear();
    lcd.setCursor(0,1);
    lcd.print("v");                     // affichage version "v300"
    if (eeConfig.vers < 100) lcd.print(" ");
    if (eeConfig.vers < 10) lcd.print(" ");
    lcd.print(eeConfig.vers);
    ///////////////////////// configuration distance zone C ou D 
    lcd.setCursor(0,0); 
    lcd.print(F("Distance zone "));
    while (digitalRead(BOK) == false) {delay(100);} // attente relachement bouton
    valeur = eeConfig.dist;             // valeur initiale
    if (valeur > 600) valeur = 600;
    eeConfig.dist = configure(valeur);
    ///////////////////////// configuration durée arret en gare   
    lcd.setCursor(0,0);
    lcd.print(F("Duree arret gare"));
    valeur = eeConfig.arret;
    if (valeur > 600) valeur = 600;
    eeConfig.arret = configure(valeur);
    ///////////////////////// configuration adresse DCC par défaut   Si adresse == 0 => mode PWM
    lcd.setCursor(0,0);
    lcd.print(F("Adresse DCC     "));
    valeur = eeConfig.adresse;
    if (valeur > 500) valeur = 500;
    eeConfig.adresse = configure(valeur);
    ///////////////////////// configuration vitesse minimale
    lcd.setCursor(0,0);
    lcd.print(F("Vitesse min     "));
    valeur = eeConfig.vmin;
    if (valeur < 2) valeur = 2;
    eeConfig.vmin = configure(valeur);
    ////////////////////////
    eeConfig.dist = 260;
    eeConfig.arret = 15;
    eeConfig.adresse = 24;
    eeConfig.vmin = 5;
    eeConfig.vers = EEPVERSION;
    lcd.setCursor(0,0);
    lcd.print(F("Enregistrement.."));
    EEPROM.put(0, eeConfig);  
    delay(1000);    
  }
}


