/**********************************************************************

DCC++ BASE STATION
COPYRIGHT (c) 2013-2016 Gregg E. Berman
With DCCpp LIBRARY
COPYRIGHT (c) 2017-2018 Thierry Paris

Version pour un Va et Vient simple et automatique sans controleur
2 GARES et 3 ZONES

COPYRIGHT (c) 2018 Dominique Bultez & Locoduino

- ARDUINO NANO
- MAIN TRACK ONLY
- NO ACCESSORIES
- NO SENSORS
- NO ETHERNET
- NO CONTROLLER
- detection automatique d'adresse DCC sur voie principale, adresses courtes et longues
- AUTOMATE va et vient 6 zones : Gare 1, A, B, C, Gare 2

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see http://www.gnu.org/licenses

**********************************************************************/

/*
Affectation des Pins du NANO sur NanoVV perso
Pin 2   <-> ENC1 encodeur pin 1
Pin 3   <-> PWM (enable, HIGH = marche, LOW = stop) pour LMD18200
Pin 4   <-> ENC2 encodeur pin 2
Pin 5   <-> detecteur Gare 1
Pin 6   <-> detecteur zone A
Pin 7   <-> detecteur zone B
Pin 8   <-> detecteur zone C
Pin 9   <-> detecteur Gare 2
Pin 10  <-> DIR (DCC = OC1B output) pour LMD18200
Pin 11  <-> Led Rouge ERREUR
Pin 12  <-> non utilisée 
Pin 13  <-> non utilisée
Pin A0  <-> Mesure de courant du Mac471
Pin A1  <-> bouton OK (encodeur)
Pin A2  <-> inter marche/arret
Pin A3  <-> inter lumière
Pin A4  <-> SDA LCD
Pin A5  <-> SLC LCD
Pin A6  iibre
Pin A7  libre
*/

/**********************************************************************
      
Version DB 300 du 20 Janvier 2018

**********************************************************************/
//#define DEBUG

const char Version[16] =  "DCCpp_VV_200118"; // 15 caractères + \0
const int EEPVERSION = 300;

#include <DCCpp.h> 
#include "Automate.h"
#include <EEPROM.h>
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x3F,16,2);  // set the LCD address to 0x27 for a 16 chars and 2 line display

ConfigEEprom eeConfig;

#include <Encoder.h>            // encodeur
Encoder myEnc(2,4);             
long oldPosition  = -999;       // encoder

#include "Configuration.h"

#define MOTOR_SHIELD_TYPE   0      // LMD18200 
#define COMM_INTERFACE   0
#define LMD_DIR 10                 // DCC_MAIN = Arduino Nano - uses OC1B - DIR LMD18200
#define DCC_MAIN 10
#define LMD_PWM 3                  // DCC_ENABLE = PWM LMD18200
#define DCC_ENABLE 3
#define Max471  A0                 // current sensor
#define gDCCStepsNumber 128        // mode DCC 128

Train mTrain(3);                   //create train default adresse 3
ZoneMonitor mzoneMonitor(PINGARE1, PINZONEA, PINZONEB, PINZONEC, PINGARE2); // create monitor for zone detectors
Automate mAutomate(0);             // creation initialisation automate
FunctionsState gLocoFunctions;     // Current functions of the loco

// LES AUTRES VARIABLES GLOBALES

unsigned long gCurrentSampleTime;
unsigned long gCurrentNulTime;
int gCurrent;
const int CurrentMax = 400;

unsigned long compteurVitesse;
bool gEtatFeux=false;          // pas de FEUX
bool gEtatStartStop=false;    // en ARRET
int  gconsigne=60; 
byte gOccupation;
bool gMesureVitesse=false;


//////////////////////////// FONCTIONS TRAIN ////////////////////////////

void Train::roule() { // t1 adresse_dcc vitesse
  DCCpp::setSpeedMain(1, adresse_dcc, 128, cran_vitesse, sens);
}

void Train::feux(bool l) {
  if (l) {
    gLocoFunctions.activate(0);
  } else {
    gLocoFunctions.inactivate(0);
  }
  DCCpp::setFunctionsMain(1, adresse_dcc, gLocoFunctions);
}

void Train::stoppe() {
  cran_vitesse = 0;
  roule();    
}

void Train::eteint() {
  lumiere=false;
  feux(lumiere);
}

void Train::allume() {
  lumiere=true;
  feux(lumiere);
}

///////////////// RAPPEL DES COORDONNEES D'AFFICHAGE LCD ///////////////////////
/* 
 *  Ligne 0 : 012 3 45 6 7 89A B CD E F
 *            @dcc  Et Er  cran  >>   *
 *  Ligne 1 : 0123 4 567 8 9AB C DEF
 *            Gar1   Cv    kmh   mA.
 *            
 * @dcc = -1 ou 24 ou 999
 * Et = Etat (0..5)  et Er = ERR (? a T + -)    
 * * = lumière
 * Lieu =  4 caractères
 * Cv = consigne en km/h
 * kmh = vitesse vraie en km/h
 */

/////// RAFRAICHISSEMENT DES AFFICHAGES PARAMETRES LOCO ///////////////////////

void lcdloco() {
  lcd.setCursor(8,0);
  if (mTrain.cran_vitesse < 100) lcd.print(" ");
  if (mTrain.cran_vitesse < 10) lcd.print(" ");
  lcd.print(mTrain.cran_vitesse); // cran
  if (mTrain.sens1vers2) {
    lcd.print(" >> ");
  } else {
    lcd.print(" << ");
  }
  if (mTrain.lumiere) {
    lcd.print("*");
  } else {
    lcd.print(" ");
  }
  lcd.setCursor(0,1);
  lcd.print(nomZone[mTrain.zone]);
  //lcd.setCursor(5,1);
  if (gconsigne < 100) lcd.print(" ");
  if (gconsigne < 10) lcd.print(" ");
  lcd.print(gconsigne);
}

void lcdETAT(char E) { // lcdETAT('a');
  lcd.setCursor(4,0);
  lcd.print(E);     // error char ? a T
}

void lcdERR(char E) { // lcdERR('+');
  lcd.setCursor(7,0);
  lcd.print(E);     // error char - +
}

void lcdCR(int n) { // compte à rebours
  lcd.setCursor(4,0);
  if (n < 100) lcd.print(" ");
  if (n < 10) lcd.print(" ");
  if (n == 0) lcd.print(" "); else lcd.print(n);
}

/////////////////////////////////////////////////////////////////////////////////////////
// RECHERCHE ADRESSE ET MISE EN ROUTE
/////////////////////////////////////////////////////////////////////////////////////////

void MiseEnRouteDCC() {
  start_DCC();
  int reponse = DCCpp::identifyLocoIdMain();
  #ifdef DEBUG
  Serial.print(F("adresse="));Serial.print(reponse);
  #endif
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(reponse);
  if (reponse == -1) {
    #ifdef DEBUG
    Serial.println(F(" >>> erreur !")); 
    #endif
    lcdETAT('?');
    mAutomate.LRclignote=true;      // clignotement Led Rouge dans mAutomate.run()
    mTrain.adresse_dcc = eeConfig.adresse;  
    delay(500);
  } else {
    #ifdef DEBUG
    Serial.println(F(" valide :-)"));
    #endif
    mAutomate.LRclignote=false;
    mTrain.adresse_dcc = reponse;
  }
  mTrain.cran_vitesse = 30;                  // vitesse 30, avant jusqu' à une gare
  mTrain.sens = true;
  mTrain.roule();
}

///////////////////////////////////////////////////////////////////////
// START / STOP DCC
///////////////////////////////////////////////////////////////////////

void start_DCC() {
  digitalWrite(DCC_ENABLE,HIGH);
  digitalWrite(LEDROUGE, HIGH);             // eteint led rouge
  lcd.setCursor(0,1);lcd.print("DCC on          ");
}

void stop_DCC() {
  digitalWrite(DCC_ENABLE,LOW);
  digitalWrite(LEDROUGE, LOW);              // allume led rouge
  lcd.setCursor(0,1);lcd.print("DCC off         ");
  delay(1000);
}


///////////////////// INITIAL SETUP ///////////////////////////////

void setup(){  

  #ifdef DEBUG
  Serial.begin(115200);           // configure serial interface
  //while (!Serial) ;             // wait for Arduino Serial Monitor
  Serial.flush();
  Serial.println(Version);
  #endif

  lcd.init();                      // initialize the lcd 
  lcd.clear();
  lcd.backlight();
  lcd.setCursor(0,0);
  lcd.print(Version);

  // Lecture de la configuration en EEPROM
  configuration();          // Configuration EEPROM lecture et ecriture si BOK appuyé
  lcd.setCursor(0,1);
  lcd.print("Demarre DCC ");      // 12
  DCCpp::begin();
  DCCpp::beginMain(UNDEFINED_PIN, LMD_DIR, LMD_PWM, Max471);        // Dc: Dir, Pwm, current sensor
  pinMode(STARTSTOP, INPUT_PULLUP);
  pinMode(LEDROUGE, OUTPUT);
  digitalWrite(LEDROUGE, HIGH);
  pinMode(LUMIERE, INPUT_PULLUP);
  pinMode(BOK, INPUT_PULLUP);

  pinMode(PINGARE1, INPUT_PULLUP);
  pinMode(PINZONEA, INPUT_PULLUP);
  pinMode(PINZONEB, INPUT_PULLUP);
  pinMode(PINZONEC, INPUT_PULLUP);
  pinMode(PINGARE2, INPUT_PULLUP);
  
  oldPosition = myEnc.read()/4;
  
  delay(500);
  digitalWrite(LEDROUGE, LOW);

  mzoneMonitor.Init();      // init zoneMonitor

  // lecture de l'état de l'inter A/M

  gEtatStartStop = !digitalRead(STARTSTOP); // start = LOW donc faut inverser
  if (gEtatStartStop) {         // position MARCHE, automate en attente de synchro en gare
    MiseEnRouteDCC();              // DCC on et roule vitesse = 30
    mAutomate.auto_etat = EXPLORE; // init automate : exploration ligne 
  } else {                      // position ARRET
    mAutomate.auto_etat = INIT; // init automate : en attente découverte et exploration ligne    
    mTrain.adresse_dcc = eeConfig.adresse;
    mTrain.cran_vitesse = 0;    // vitesse = 0
    mTrain.sens = true;
    lcd.clear();
  }
  gconsigne = 60;             // valeur initial de la consigne de vitesse et de l'encoder
  mTrain.consigne = gconsigne;
  mTrain.cran_consigne = gconsigne;

  // lecture de l'état de l'inter LUMIERE
  
  gEtatFeux = !digitalRead(LUMIERE);
  if (gEtatFeux) {            // true = allumé
    mTrain.allume();          // lumiere ON
  } else {                    // false = eteint
    mTrain.eteint();          // lumiere OFF
  }
  lcd.setCursor(0,0);
  lcd.print(mTrain.adresse_dcc);// ligne 0 : adresse
  lcd.print(" ");               // effacement  du "1" en cas d'erreur
  lcdloco();                    // ligne 1 : vitesse, sens, lumiere et consigne
  
  // initialisation des états de l'automate
  mAutomate.auto_ERR = false;
  mAutomate.timeout = millis();          // armement de la tempo "timeout"
  mzoneMonitor.zoneTime = millis();
  if (mAutomate.auto_etat == EXPLORE) mTrain.roule();

} // setup

//////////////////////// MAIN ARDUINO LOOP /////////////////////

void loop() {

  ///////////////////////// DCCpp task /////////////////////////
  
  DCCpp::loop();
  
  /////// Affichage courant, détection de court-circuit et de faux contact //////
  
  // affichage courant = la valeur maxi pendant 500 ms
  if ((millis() - gCurrentSampleTime) > 500) {
    gCurrentSampleTime = millis();
    lcd.setCursor(13,1);lcd.print(gCurrent*5);lcd.print(' '); // en mA réels
    gCurrent = 0;
  }
  int iCurrent = analogRead(A0);
  if (iCurrent > gCurrent) {
    gCurrent = iCurrent;
    if (gCurrent > CurrentMax) { // 400 * 5 = 2000 mA
    stop_DCC();
    } 
  }
  if ((iCurrent == 0) && (mAutomate.auto_etat > 2)) { // perte de consommation 
    if ((millis() - gCurrentNulTime) > NULCURRENT_TIME) {
      mAutomate.auto_etat=INIT;           // RAZ automate
      mAutomate.LRclignote=true;          // led rouge clignote
      mTrain.cran_vitesse = 0;            // vitesse = 0
      mTrain.lumiere=false;
      delay(2000);
      stop_DCC();                         // arrêt DCC
      lcd.setCursor(10,1);lcd.print("Faucon");
      // panne de courant : faire STOP puis START pour redémarrer
    }   
  } else {
    gCurrentNulTime = millis();           // relance tempo
  }

  ///////////////////// détections de zones /////////////////////
  
  if (mzoneMonitor.checkZoneTime()) {             // test occupations toutes les 0,1s
    byte loccupation = mzoneMonitor.checkZone(); // détection seulement des changements de type occupation
    byte test = loccupation ^ gOccupation;        // un bit 1 marque chaque différence
    if (test != 0) {                              // on n'en traite qu'une seule à la fois
      gOccupation = loccupation;                  // sauve valeur de gOccupation
      test = test & loccupation;                  // valeur du changement détecté (soit 1 soit 0)
      switch (test) {                             // on ne traite qu'une nouvelle occupation
        case 1:      //ZGARE1
        _doGare1();
        break;
        case 2:      //ZONEA
        _doZoneA();
        break;
        case 4:      //ZONEB
        _doZoneB();
        break;
        case 8:      //ZONEC
        _doZoneC();
        break;
        case 16:     //ZGARE2
        _doGare2();
        break;
      }
    }
  }   //   fin de mzoneMonitor.checkZoneTime()

  //////// évolution des états temporels de l'automate /////////////
    
  if (mAutomate.auto_seconde) {                   // TOP seconde écoulée !
    mAutomate.auto_seconde=false;
    mAutomate.timeseconde = millis();             // réarmement de la tempo "seconde"
    lcdETAT('a');
    lcd.print(mAutomate.auto_etat);               // a+ N° etat automate
    switch (mAutomate.auto_etat) {                // phases automate indépendantes des tests d'occupation
      case INIT:   
      // inactif   
      break;
      case EXPLORE:   
      // roule vers gare                          // attente détection zone ralentissement
      break;
      case ARRET:                                 // ATTENTE EN GARE et départ si temps écoulé
      mAutomate.arretengare--;                    // decrementé 1 fois par seconde
      lcdCR(mAutomate.arretengare);               // affichage du temps restant
      if (mAutomate.arretengare == 0) {           // fin de la tempo
        mTrain.cran_vitesse = 10;                 // demarrage du train cran 10
        mTrain.roule();
        if (gEtatFeux) mTrain.allume();
        mAutomate.auto_etat=ACCELERE;             // etat : accélération
      }
      break;
      case ACCELERE :                             // ACCELERATION du train
      mTrain.cran_vitesse+=mTrain.acceleration;             
      if (mTrain.cran_vitesse > mTrain.cran_consigne){ // atteinte de la consigne de vitesse
        mTrain.cran_vitesse = mTrain.cran_consigne;
        mAutomate.auto_etat=PALIER;               // etat : palier à vitesse constante
      }
      mTrain.roule();
      break;  
      case PALIER :                               // ROULE a vitesse contante
      if (mTrain.cran_vitesse != mTrain.cran_consigne) { 
        mTrain.cran_vitesse = mTrain.cran_consigne; 
        mTrain.roule();                           // attente détection zone ralentissement
      }
      break;
      case RALENTI :                              // RALENTISSEMENT du train
      if (mTrain.sens1vers2) {
        mTrain.cran_vitesse-=mTrain.deceleration2;// Vers gare 2
      } else {
        mTrain.cran_vitesse-=mTrain.deceleration1;// Vers gare 1
      }
      if (mTrain.cran_vitesse < eeConfig.vmin) {    // atteinte de la consigne minimale avant Gare
        mTrain.cran_vitesse = eeConfig.vmin;
        if (mTrain.troplent==false) {
          mTrain.troplent=true;
          if (mTrain.sens1vers2) {
            mzoneMonitor.zoneTimeCG2 = millis();    // comptage du temps à vitesse MIN de la zone  à la gare 2
          } else {
            mzoneMonitor.zoneTimeAG1 = millis();    // comptage du temps à vitesse MIN de la zone A à la gare 1
          } 
        }
      }
      mTrain.roule();
      break;    
    }  // switch mAutomate       
    lcdloco();
  }  // toutes les secondes

  ///////////////// Gestion des temporisations de l'automate //////////////////
  
  mAutomate.runauto();                            // fait les choses liées au temps 
  
  if (mAutomate.auto_ERR) {                       // TIMEOUT : défaut de détection de zone
    lcdETAT('T');
    lcd.print(mAutomate.auto_etat);               // TX ERR.
    mAutomate.LRclignote=true;                    // clignotement Led Rouge dans mAutomate.run()
    lcdloco();
    mAutomate.auto_ERR=false;
    mAutomate.timeout = millis();                 // rearmement de la tempo "timeout"
  }

  //////////////// ENCODEUR GESTION DE LA CONSIGNE DE VITESSE ////////////////////////
    
  long newPosition = myEnc.read()/4;
    if (newPosition != oldPosition) {  
      if ( newPosition > oldPosition) gconsigne++;  else  gconsigne--;
      oldPosition = newPosition; 
      if (gconsigne < 0) gconsigne=0;
      if (gconsigne > 180) gconsigne=180;
      mTrain.consigne = gconsigne;
    }
 
  //////////////////////////// inter MARCHE / ARRET ////////////////////////
    
  if (gEtatStartStop == digitalRead(STARTSTOP)) {   // changement d'etat
    gEtatStartStop = !digitalRead(STARTSTOP); // toujours inverser
    #ifdef DEBUG
    Serial.print(F("SS="));Serial.println(gEtatStartStop);
    #endif
    if (gEtatStartStop) {                  // start
      MiseEnRouteDCC();
      mzoneMonitor.zoneTime = millis();
      mAutomate.auto_etat=EXPLORE;        // init automate : attente arrivée en gare a vitesse constante    
      mAutomate.auto_ERR = false;
      mAutomate.timeout = millis();       // armement de la tempo "timeout"
    } else {        // stop
      mAutomate.auto_etat=INIT;           // RAZ automate
      mAutomate.LRclignote=false;         // au cas où elle clignote
      mTrain.cran_vitesse = 0;            // vitesse = 0
      mTrain.lumiere=false;
      mTrain.feux(false);
      mTrain.roule();
      delay(2000);
      stop_DCC();                         // arrêt DCC
    }
    lcdloco();                          // affichage lcd
  }

  //////////////////////////// Bouton LUMIERE ////////////////////////
    
  if (gEtatFeux == digitalRead(LUMIERE)) {
    gEtatFeux = !digitalRead(LUMIERE);
    if (gEtatFeux) { 
      mTrain.allume();   // lumiere ON
    } else {
      mTrain.eteint();   // lumiere OFF
    }
    lcdloco();           // mise a jour affichage
  }
  
} // loop

void _doGare1() {
  mTrain.zone = ZGARE1;
  //mAutomate.auto_position = ZGARE1;
  mAutomate.auto_ERR=false;
  mAutomate.timeout = millis();                 // rearmement de la tempo "timeout"
  if (mAutomate.auto_etat == RALENTI) {         // arrivée en mode RALENTI
    mzoneMonitor.zoneTimeA = (millis() - mzoneMonitor.zoneTimeA)/1000; //durée ZONEA en secondes
    // si la vitesse est supérieure à la Vitesse MIN, on augmente mTrain.deceleration1
    if (mTrain.cran_vitesse > (eeConfig.vmin))  { // le train rentre en gare trop vite             
      mTrain.deceleration1 = mTrain.deceleration1 + ((mTrain.cran_vitesse - eeConfig.vmin)/mzoneMonitor.zoneTimeA );
      lcdERR('+');
    }
    if (mTrain.troplent) {                                    // trop lent donc diminuer la deceleration1
      mzoneMonitor.zoneTimeAG1 = millis() - mzoneMonitor.zoneTimeAG1; //en millisecondes
      if (mzoneMonitor.zoneTimeAG1 > 2000) { // il faut ralentir moins vite
        mTrain.deceleration1--;
        if (mTrain.deceleration1 < 1) mTrain.deceleration1 = 1;
        lcdERR('-');
      }
    } // fin regulation
    mTrain.stoppe();                          // arrêt
    //mTrain.eteint();                        // extinction des feux
    mAutomate.arretengare=eeConfig.arret;
    mAutomate.auto_etat=ARRET;                // etat = decomptage pendant l'arrêt en gare
    mTrain.sens1vers2 = true;                 // vers la gare 2
    mTrain.sens = !mTrain.sens;               // inversion du sens DCC
  } // fin RALENTI
  lcdloco();  
  if (mAutomate.auto_etat == EXPLORE) {     // Arrivée en mode EXPLORE 
    mTrain.stoppe();                        // arrêt
    mAutomate.arretengare=eeConfig.arret;   // tempo d'arret en gare en secondes
    mAutomate.auto_etat=ARRET;              // etat = decomptage pendant l'arrêt en gare
    mTrain.sens1vers2 = true;               // vers le gare 2
    mTrain.sens = !mTrain.sens;             // inversion du sens DCC
  }  
}

void _doZoneA() {
  mTrain.zone = ZONEA;
  // calcul de Vitesse seulement dans le sens G2 vers G1 ---1 cm/s <-> 5,76 km/h 
  if (gMesureVitesse && !mTrain.sens1vers2) {  // vers Gare 1
    compteurVitesse=millis()-compteurVitesse;
    unsigned long VKM = (eeConfig.dist*5760)/compteurVitesse;
    mTrain.vitesseVraie=(int)(VKM);
    lcd.setCursor(9,1);
    if (mTrain.vitesseVraie < 100) lcd.print(" ");
    if (mTrain.vitesseVraie < 10) lcd.print(" ");
    lcd.print(mTrain.vitesseVraie);
    gMesureVitesse=false;
    // ajustement de la vitesse mTrain.cran_consigne
    if ((mTrain.vitesseVraie >0) && (mTrain.vitesseVraie <160)) {
      if ((mTrain.vitesseVraie > mTrain.consigne + 2) || (mTrain.vitesseVraie < mTrain.consigne - 2)) {
        mTrain.cran_consigne = mTrain.cran_consigne - ((mTrain.vitesseVraie-mTrain.consigne)/2);
        if (mTrain.cran_consigne < eeConfig.vmin) mTrain.cran_consigne = eeConfig.vmin;
      }
    }
  }
  // ralentissement si automate = PALIER et seulement dans le sens 2 vers 1         
  if (!mTrain.sens1vers2 && (mAutomate.auto_etat == PALIER)) {  
    mAutomate.auto_etat=RALENTI;            //etat = ralentissement
    mTrain.troplent=false;
    mAutomate.auto_seconde=false;
    mAutomate.timeseconde = millis();       // armement de la tempo "seconde"
    mzoneMonitor.zoneTimeA = millis();      // comptage de la traversée de la zone A
  }
  lcdERR(' ');    
}
        
void _doZoneB() {
  mTrain.zone = ZONEB;
  //mAutomate.auto_position = ZONEB;
  compteurVitesse=millis();                 // lancement compteur de vitesse
  gMesureVitesse=true;   
}
        
void _doZoneC() {
  mTrain.zone = ZONEC;
  //mAutomate.auto_position = ZONEC;
  // calcul Vitesse seulement dans le sens 1 vers 2 ----1 cm/s <-> 5,76 km/h 
  if (gMesureVitesse && mTrain.sens1vers2) {
    compteurVitesse=millis()-compteurVitesse;
    unsigned long VKM = (eeConfig.dist*5760)/compteurVitesse;
    mTrain.vitesseVraie=(int)(VKM);
    lcd.setCursor(9,1);
    if (mTrain.vitesseVraie < 100) lcd.print(" ");
    if (mTrain.vitesseVraie < 10) lcd.print(" ");
    lcd.print(mTrain.vitesseVraie);
    gMesureVitesse=false;
    // ajustement de la vitesse mTrain.cran_consigne
    if ((mTrain.vitesseVraie >0) && (mTrain.vitesseVraie <160)) {
      if ((mTrain.vitesseVraie > mTrain.consigne + 2) || (mTrain.vitesseVraie < mTrain.consigne - 2)) {
        mTrain.cran_consigne = mTrain.cran_consigne - ((mTrain.vitesseVraie-mTrain.consigne)/2);
        if (mTrain.cran_consigne < eeConfig.vmin) mTrain.cran_consigne = eeConfig.vmin;
      }
    }
  }       
  // ralentissement si automate = PALIER dans le sens 1 vers 2   
  if (mTrain.sens1vers2 && (mAutomate.auto_etat == PALIER)) {  
    mAutomate.auto_etat=RALENTI;          // etat = ralentissement
    mTrain.troplent=false;
    mAutomate.auto_seconde=false;
    mAutomate.timeseconde = millis();     // armement de la tempo "seconde"
    mzoneMonitor.zoneTimeC = millis();    // comptage de la traversée de la zone C
  }
  lcdERR(' ');          
}
        
void _doGare2() {
  mTrain.zone = ZGARE2;  
  //mAutomate.auto_position = ZGARE2;
  mAutomate.auto_ERR=false;
  mAutomate.timeout = millis();             // rearmement de la tempo "timeout"
  if (mAutomate.auto_etat == RALENTI) {     // Mode RALENTI
    // si la vitesse est supérieure à la Vitesse MIN, on augmente mTrain.deceleration2
    mzoneMonitor.zoneTimeC = (millis() - mzoneMonitor.zoneTimeC)/1000; //en secondes
    if (mTrain.cran_vitesse > (eeConfig.vmin)) { // le train rentre en gare trop vite              
      mTrain.deceleration2 = mTrain.deceleration2 + ((mTrain.cran_vitesse - eeConfig.vmin)/++mzoneMonitor.zoneTimeC);
      lcdERR('+');
    }
    if (mTrain.troplent) {                     // sinon trop lent vite donc diminuer la deceleration2
      mzoneMonitor.zoneTimeCG2 = millis() - mzoneMonitor.zoneTimeCG2; //en millisecondes
      if (mzoneMonitor.zoneTimeCG2 > 2000) {
        mTrain.deceleration2--;
        if (mTrain.deceleration2 < 1) mTrain.deceleration2 = 1;
        lcdERR('-');
      }
    } // fin regulation    
    mTrain.stoppe();                        // arrêt
    mAutomate.arretengare=eeConfig.arret;   // tempo d'arret en gare en secondes
    mAutomate.auto_etat=ARRET;              // etat = decomptage pendant l'arrêt en gare
    mTrain.sens1vers2 = false;              // vers le gare 1
    mTrain.sens = !mTrain.sens;             // inversion du sens DCC
  } // fin RALENTI
  if (mAutomate.auto_etat == EXPLORE) {     // Mode EXPLORE
    mTrain.stoppe();                        // arrêt
    mAutomate.arretengare=eeConfig.arret;   // tempo d'arret en gare en secondes
    mAutomate.auto_etat=ARRET;              // etat = decomptage pendant l'arrêt en gare
    mTrain.sens1vers2 = false;              // vers le gare 1
    mTrain.sens = !mTrain.sens;             // inversion du sens DCC
  }
  lcdloco();  
}
        

